package com.mints.street.netwrok.common

import android.text.TextUtils
import android.util.Log
import com.bytedance.hume.readapk.HumeSDK
import com.google.gson.Gson
import com.mints.street.AppApplication
import com.mints.street.BuildConfig
import com.mints.street.bean.AppRequest
import com.mints.street.manager.UserManager
import com.mints.street.utils.CommonUtils
import com.mints.street.utils.encry.AESUtils
import com.mints.street.utils.encry.Base64
import com.mints.street.utils.encry.MD5
import okhttp3.*
import okio.Buffer
import org.json.JSONObject
import java.io.IOException
import java.io.UnsupportedEncodingException
import java.nio.charset.Charset

/**
 * 拦截器
 */
class OkHttpInterceptor(aesKey: String) : Interceptor {
    private var aesKey = ""

    /**
     * 实例化拦截器对象
     */
    init {
        this.aesKey = aesKey
    }

    @Throws(IOException::class)
    override fun intercept(chain: Interceptor.Chain): Response {
        val tokenID: String = UserManager.INSTANCE.getTokenID()
        val time = System.currentTimeMillis()
        val channel = createChannel()
        var request = chain.request()
        // 加密
        request = encrypt(request, tokenID, time, channel)
        // header
        val builder = addHeader(request, tokenID, time, channel)
        // 解密
        var response = chain.proceed(builder)
        response = decrypt(response)
        return response
    }

    /**
     * 创建渠道
     *
     * @return
     */
    private fun createChannel(): String {
        var channel: String = CommonUtils.getAppMetaData(AppApplication.getContext(), "CHANNEL_NAME")
        try {
            // 穿山甲分包渠道
            val ttChannel = HumeSDK.getChannel(AppApplication.getContext())
            if (!TextUtils.isEmpty(ttChannel)) {
                // 此处方式不能修改，已和后端约定
                channel = channel + "_" + ttChannel
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return channel
    }

    /**
     * 添加header
     *
     * @param request
     * @param tokenID
     * @param time
     * @param channel
     * @return
     */
    @Throws(UnsupportedEncodingException::class)
    private fun addHeader(request: Request, tokenID: String, time: Long, channel: String): Request {
        return request.newBuilder()
                .addHeader("version", BuildConfig.VERSION_NAME)
                .addHeader("token", tokenID)
                .addHeader("channel", channel)
                .addHeader("new-session", MD5.GetMD5Code(time.toString()))
                .addHeader("last-session", Base64.encode(time.toString().toByteArray(charset("UTF-8"))))
                .build()
    }

    /**
     * json加密
     *
     * @param request
     * @param tokenID
     * @param time
     * @param channel
     * @return
     * @throws IOException
     */
    @Throws(IOException::class)
    private fun encrypt(request: Request, tokenID: String, time: Long, channel: String): Request {
        //获取请求body，只有@Body 参数的requestBody 才不会为 null
        var request = request
        val requestBody = request.body
        if (requestBody != null) {
            val buffer = Buffer()
            requestBody.writeTo(buffer)
            var charset = Charset.forName("UTF-8")
            val contentType = requestBody.contentType()
            if (contentType != null) {
                charset = contentType.charset(charset)
            }

            // 原始报文
            var valueStr = buffer.readString(charset!!)
            val checkStr = valueStr

            //加密
            if (isSign(request) && !TextUtils.isEmpty(valueStr)) {
                valueStr = AESUtils.encrypt(valueStr, aesKey)
            }

            // 渠道
            val channelName = "android_$channel"

            // sign
            val requestSign = "$tokenID:$channelName:$time"
            val sign = MD5.GetMD5Code(requestSign)
            // 验证
            val check = MD5.GetMD5Code("$requestSign:$checkStr")
            val appRequest = AppRequest(channelName, sign, check, valueStr)


            val body: RequestBody = RequestBody.create(contentType, Gson().toJson(appRequest))
            request = request.newBuilder()
                    .post(body)
                    .build()
        }
        return request
    }

    /**
     * json解密
     *
     * @param response
     * @return
     * @throws IOException
     */
    @Throws(IOException::class)
    private fun decrypt(response: Response): Response {
        var response = response
        if (response.isSuccessful && isSign(response.request)) {
            //the response data
            val body = response.body
            val source = body!!.source()
            source.request(Long.MAX_VALUE) // Buffer the entire body.
            val buffer = source.buffer()
            var charset = Charset.defaultCharset()
            val contentType = body.contentType()
            if (contentType != null) {
                charset = contentType.charset(charset)
            }
            var rspString: String? = buffer.clone().readString(charset!!)
            // 解密
            try {
                val json = JSONObject(rspString)
                if (!json.isNull("data")) {
                    val data = json["data"]
                    rspString = AESUtils.detrypt(data.toString(), aesKey)
                    val jsonArray = JSONObject(rspString)
                    json.put("data", jsonArray)
                    rspString = json.toString()
                    // rspString = json.toString().replace("\\", "");
                }
            } catch (e: Exception) {
                Log.d("OkHttpInterceptor", "json解密失败。url：${response.request.url}响应报文：${buffer.clone().readString(charset)}")
                e.printStackTrace()
            }
            val responseBody = ResponseBody.create(contentType, rspString!!)
            response = response.newBuilder().body(responseBody).build()
        }
        return response
    }

    /**
     * 是否加/解密 - 接口非common都加/解密
     *
     * @param request
     * @return true=加/解密
     */
    private fun isSign(request: Request): Boolean {
        val url = request.url.toString()
        //是否加密标识
        var isSign = true
        if (!TextUtils.isEmpty(url)) {
            isSign = !url.contains("common/")
        }
        return isSign
    }


}