package com.fry.base.expand

import android.graphics.Color
import android.graphics.Typeface
import android.text.*
import android.text.method.LinkMovementMethod
import android.text.style.*
import android.view.View
import android.widget.TextView

/**
 * 2019-12-01 created
 * @author KOM
 */

interface LinkCallback {
    fun getTextView(): TextView
    fun onClick(view: View)
}

/**
 * 更新文本样式
 */
data class TextSpanStyle(
        var text: String,
        /**
         * 字体大小 px
         */
        var fontSize: Int? = null,
        /**
         * 字体颜色
         */
        var fontColor: Int? = null,
        /**
         * 字体样式
         * 正常[Typeface.NORMAL] 粗体[Typeface.BOLD] 斜体[Typeface.ITALIC] 粗斜体[Typeface.BOLD_ITALIC]
         */
        var typeface: Int? = null,
        /**
         * 背景颜色
         */
        var backgroundColor: Int? = null,
        /**
         * 删除线
         */
        var strikeThrough: Boolean? = null,
        /**
         * 下划线
         */
        var underline: Boolean? = null,
        /**
         * 下标
         */
        var subscript: Boolean? = null,
        /**
         * 上标
         */
        var superscript: Boolean? = null,
        /**
         * 链接跳转
         */
        var link: LinkCallback? = null
)

fun Spannable.updateTextStyle(
        allText: String, target: TextSpanStyle
): Spannable {
    if (!TextUtils.isEmpty(allText) && target.text.isNotEmpty()) {
        if (allText.contains(target.text)) {
            val start = allText.indexOf(target.text)
            val end = start + target.text.length
            target.fontColor?.apply {
                setSpan(ForegroundColorSpan(this), start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
            target.fontSize?.apply {
                setSpan(AbsoluteSizeSpan(this, false), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
            }
            target.typeface?.apply {
                setSpan(StyleSpan(this), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
            target.backgroundColor?.apply {
                setSpan(BackgroundColorSpan(this), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
            target.strikeThrough?.apply {
                if (!this) {
                    return@apply
                }
                setSpan(StrikethroughSpan(), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
            target.underline?.apply {
                if (!this) {
                    return@apply
                }
                setSpan(UnderlineSpan(), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
            target.subscript?.apply {
                if (!this) {
                    return@apply
                }
                setSpan(SubscriptSpan(), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
            target.superscript?.apply {
                if (!this) {
                    return@apply
                }
                setSpan(SuperscriptSpan(), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            }
            target.link?.apply {
                // 移除NoCopySpan 因为在开启无障碍服务后，NoCopySpan会导致崩溃，具体原因见如下链接
                // https://yrom.net/blog/2019/12/03/crashed-on-accessibilitynodeInfo-settext-in-textview/
                setSpan(object : ClickableSpan() {
                    override fun onClick(widget: View) {
                        target.link?.onClick(widget)
                    }

                    override fun updateDrawState(ds: TextPaint) {
                        // 默认使用 TextSpanStyle 的字体颜色，未设置时，使用TextView的linkColor
                        ds.color = target.fontColor ?: ds.linkColor
                        // 默认没有下划线
                        ds.isUnderlineText = target.underline ?: false
                    }
                }, start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
                // 配置链接方法，不配置ClickableSpan点击事件不会被触发
                this.getTextView().movementMethod = LinkMovementMethod.getInstance()
                // 设置点击后的颜色为透明
                this.getTextView().highlightColor = Color.TRANSPARENT
            }
        }
    }
    return this
}

fun String.updateTextStyle(target: TextSpanStyle): Spannable {
    return let { allText ->
        SpannableString(allText).updateTextStyle(allText, target)
    }
}