package com.fry.base.expand

import android.content.res.ColorStateList
import android.graphics.drawable.Drawable
import android.util.TypedValue
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.annotation.CallSuper
import androidx.annotation.ColorInt
import okio.JvmOverloads
import java.io.Serializable

/**
 * 2020-01-05 created
 * @author KOM
 */

interface ViewAttribute<V : View> : Function1<V, V>, Serializable

/**
 * 单位px
 * @param left 若为null，则使用原来的内边距
 * @param top 若为null，则使用原来的内边距
 * @param right 若为null，则使用原来的内边距
 * @param bottom 若为null，则使用原来的内边距
 */
class Padding constructor(
        val left: Int? = null, val top: Int? = null,
        val right: Int? = null, val bottom: Int? = null
) : ViewAttribute<View> {

    constructor(horizontal: Int?, vertical: Int?)
            : this(horizontal, vertical, horizontal, vertical)

    constructor(all: Int?)
            : this(all, all, all, all)

    override fun invoke(view: View): View {
        view.setPadding(
                left ?: view.paddingStart,
                top ?: view.paddingTop,
                right ?: view.paddingEnd,
                bottom ?: view.paddingBottom
        )
        return view
    }

}

/**
 * 单位px
 * @param left 若为null，则使用原来的外边距
 * @param top 若为null，则使用原来的外边距
 * @param right 若为null，则使用原来的外边距
 * @param bottom 若为null，则使用原来的外边距
 */
class Margin constructor(
        val left: Int? = null, val top: Int? = null,
        val right: Int? = null, val bottom: Int? = null
) : ViewAttribute<View> {

    constructor(horizontal: Int?, vertical: Int?)
            : this(horizontal, vertical, horizontal, vertical)

    constructor(all: Int?)
            : this(all, all, all, all)

    override fun invoke(view: View): View {
        if (view.layoutParams is ViewGroup.MarginLayoutParams) {
            with(view.layoutParams as ViewGroup.MarginLayoutParams) {
                marginStart = left ?: marginStart
                topMargin = top ?: topMargin
                marginEnd = right ?: marginEnd
                bottomMargin = bottom ?: bottomMargin
            }
        }
        return view
    }
}

class Background @JvmOverloads constructor(
        val drawable: Drawable? = null,
        val color: Int? = null
) : ViewAttribute<View> {

    override fun invoke(view: View): View {
        drawable?.apply {
            view.background = this
        } ?: color?.apply {
            view.setBackgroundColor(this)
        }
        return view
    }
}

open class ViewStyle<V : View> @JvmOverloads constructor(
        var padding: Padding? = null,
        var margin: Margin? = null,
        var background: Background? = null
) : ViewAttribute<V> {

    @CallSuper
    override fun invoke(view: V): V {
        background?.invoke(view)
        padding?.invoke(view)
        margin?.invoke(view)
        return view
    }
}

class TextStyle @JvmOverloads constructor(
        var text: CharSequence?, var size: Float? = null,
        @ColorInt var color: Int? = null, background: Background? = null,
        padding: Padding? = null, margin: Margin? = null
) : ViewStyle<TextView>(padding, margin, background) {

    override fun invoke(view: TextView): TextView {
        super.invoke(view)
        with(view) {
            text = this@TextStyle.text ?: text
            setTextSize(
                    TypedValue.COMPLEX_UNIT_PX,
                    this@TextStyle.size?.sp2px()?.toFloat() ?: textSize
            )
            this@TextStyle.color?.apply {
                setTextColor(ColorStateList.valueOf(this))
            }
        }
        return view
    }
}

fun View.isVisible() = visibility == View.VISIBLE