package com.fry.base.recycler_view

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.CallSuper
import androidx.annotation.LayoutRes
import androidx.databinding.Bindable
import androidx.databinding.DataBindingUtil
import androidx.databinding.ObservableInt
import androidx.databinding.ViewDataBinding
import androidx.recyclerview.widget.RecyclerView
import kotlin.math.abs

/**
 * 2020/6/17 created
 *
 * @author KOM
 *
 * [VHModel]
 * 1. 模型实体 一个VHModel类对应一个Style的Layout xml文件
 * 2. 插入xml视图文件中的对象(A)的Class
 *
 * 不允许直接在VHModel中保存getView()返回的View去直接操作View
 * 因为RecyclerView的缓存回收机制，[VH]在被回收后,
 * 再重新给[VHModel]分配[VH]时，很有可能就不是上一个[VH]，
 * 强烈建议使用双向数据绑定去处理[VHModel]与UI的交互
 *
 * [VHMAdapter]
 * 装载[VHModel]的[RecyclerView.Adapter]
 */
abstract class VHModel {
    var isBinding: Boolean = false
        private set

    /**
     * 这里不能使用[Bindable]
     * 因为position属性在不同的xml文件中代表的BR.position的值不一定一样,
     * BR的ID生成规则是:
     * 1. 模块中所有变量名根据ASCII码升序排序
     * 2. 将排序好的变量从1生成BR id(0是_all变量的值，用于更新整个Item)
     * 如:
     * 模块中使用了两个变量A,B;则他们的BR id分别为1,2
     */
    var position = ObservableInt(-1)
        internal set

    /**
     * 创建视图
     * View 与 VH Object 是一对一关系
     * View 与 VHModel Object 是一对多关系
     */
    @LayoutRes
    abstract fun getLayoutId(): Int

    /**
     * 绑定到XML中的[VHModel]对象的变量名
     */
    abstract fun getVariableId(): Int?

    /**
     * VHModel 与 VH 绑定的时候调用
     * 注意:
     * 在[onBind]方法执行时，数据还没有与View绑定完成，
     * 所以，这里通过binding获取到的数据不是通过数据绑定绑定的数据
     */
    @CallSuper
    open fun onBind(binding: ViewDataBinding) {
        isBinding = true
        getVariableId()?.let { variableId ->
            if (binding.setVariable(variableId, this)) {
                binding.executePendingBindings()
            } else {
                val context = binding.root.context
                val resources = context.resources
                val layoutName = resources.getResourceName(getLayoutId())
                val bindingVariableName = DataBindingUtil.convertBrIdToString(variableId)
                throw IllegalStateException("Could not bind variable '$bindingVariableName' in layout '$layoutName'")
            }
        }
    }

    /**
     * VHModel 与 VH 解绑时调用
     */
    @CallSuper
    open fun onUnBind() {
        isBinding = false
    }
}

/**
 * ViewHolder视图占位类
 */
class VH<Model : VHModel>(view: View) :
        RecyclerView.ViewHolder(view) {

    private val binding = DataBindingUtil.bind<ViewDataBinding>(view)!!
    private var model: Model? = null
    private var isBinding: Boolean = false

    fun onBind(newModel: Model) {
        val oldModel = model
        oldModel?.onUnBind()
        newModel.onBind(binding)
        model = newModel
        isBinding = true
    }

    fun onUnBind() {
        model?.onUnBind()
        model = null
        isBinding = false
    }

}


class VHMAdapter<Model : VHModel>(
        data: MutableList<Model>? = null
) : RecyclerView.Adapter<VH<Model>>(), View.OnAttachStateChangeListener {

    private var layoutInflater: LayoutInflater? = null

    private val models = mutableListOf<Model>().apply {
        data?.let {
            restPosition(it)
            addAll(it)
        }
    }

    private var oldItemCount: Int = models.size

    /**
     * key：ViewHolder Type
     * value: position
     */
    private val typePositionCache = LinkedHashMap<Int, Int>()

    /**
     * key：position
     * value: ViewHolder 唯一标识
     */
    private val positionAndVHUniqueCache = LinkedHashMap<Int, VHUnique>()

    override fun getItemCount(): Int = models.size

    //--------------------------------------//
    //----------------多布局----------------//
    //------------------------------------//

    override fun getItemViewType(position: Int): Int {
        val model = models[position]
        val vhUnique = VHUnique(model.javaClass.simpleName, model.getLayoutId())
        if (isUpdated(position, vhUnique)) {
            typePositionCache.clear()
            positionAndVHUniqueCache.clear()
        }
        val type = abs(vhUnique.hashCode())
        typePositionCache[type] = position
        positionAndVHUniqueCache[position] = vhUnique
        return type
    }

    /**
     * [models]是否已经被更新
     * 1. ViewHolder 数量改变
     * 2. 相同位置的数据类型改变
     */
    private fun isUpdated(position: Int, vhUnique: VHUnique): Boolean {
        val oldVHUnique = positionAndVHUniqueCache[position]
        if (oldItemCount != itemCount || oldVHUnique != vhUnique) {
            oldItemCount = itemCount
            return true
        }
        return false
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): VH<Model> {
        layoutInflater = layoutInflater ?: LayoutInflater.from(parent.context)
        parent.addOnAttachStateChangeListener(this)
        return VH<Model>(
                layoutInflater!!.inflate(
                        models[typePositionCache[viewType]!!].getLayoutId(),
                        parent,
                        false
                )
        )
    }

    override fun onBindViewHolder(holder: VH<Model>, position: Int) {
        holder.onBind(models[position])
    }

    //--------------------------------------//
    //-------------解绑VHModel--------------//
    //------------------------------------//

    override fun onViewRecycled(holder: VH<Model>) {
        super.onViewRecycled(holder)
        holder.onUnBind()
    }

    override fun onViewAttachedToWindow(v: View?) {
        if (v is RecyclerView) {
            v.addOnAttachStateChangeListener(this)
        }
    }

    /**
     * 当最后一条可见,当又没显示全时,RecyclerView会缓存一条ViewHolder,
     * 这时缓存的这条ViewHolder已经绑定,但ViewHolder的itemView并未attachedToWindow,
     * 此时如果界面销毁,就触发不了itemView的onViewAttachedToWindow从而解绑VHModel,
     * So,利用RecyclerView的onViewDetachedFromWindow解绑VHModel
     */
    override fun onViewDetachedFromWindow(v: View?) {
        models.filter {
            it.isBinding
        }.forEach {
            it.onUnBind()
        }
        if (v is RecyclerView) {
            v.removeOnAttachStateChangeListener(this)
        }
    }

    //-------------------------------------//
    //--------------增删改查Api--------------//
    //------------------------------------//

    fun getAll(): List<Model> {
        return models
    }

    fun add(model: Model, isNotify: Boolean = true) {
        insertToLast(model, isNotify)
    }

    fun addAll(models: Iterable<Model>, isNotify: Boolean = true) {
        insertAll(models, position = -1, isNotify = isNotify)
    }

    fun find(position: Int): Model? {
        if (position < 0 || position >= models.size) {
            return null
        }
        return models[position]
    }

    fun find(model: Model): Model? {
        return models.firstOrNull { model == models }
    }

    fun insert(model: Model, position: Int, isNotify: Boolean = true): Boolean {
        val size = models.size
        if (position < 0 || position > size) {
            return false
        }
        models.add(position, model)
        restPosition()
        if (isNotify) {
            notifyItemInserted(position)
        }
        return true
    }

    /**
     * @param position 插入数据的位置 -1 表示从队尾插入 0 表示从队头插入
     */
    fun insertAll(models: Iterable<Model>, position: Int = -1, isNotify: Boolean = true): Boolean {
        var insertPosition = position
        if (insertPosition == -1) {
            insertPosition = this.models.size
        }
        if (insertPosition < 0 || insertPosition > this.models.size) {
            return false
        }
        if (!models.iterator().hasNext()) {
            return false
        }
        var size = 0
        models.forEachIndexed { index, model ->
            this.models.add(insertPosition + index, model)
            size++
        }
        restPosition()
        if (isNotify) {
            notifyItemRangeInserted(insertPosition, insertPosition + size)
        }
        return true
    }

    fun insertToFirst(model: Model, isNotify: Boolean = true) =
            insert(model, 0, isNotify)

    fun insertToLast(model: Model, isNotify: Boolean = true) =
            insert(model, models.size, isNotify)

    fun replace(model: Model, position: Int, isNotify: Boolean = true): Boolean {
        if (position < 0 || position > models.size - 1) {
            return false
        }
        models.removeAt(position)
        models.add(position, model)
        restPosition()
        if (isNotify) {
            notifyItemChanged(position)
        }
        return true
    }

    /**
     * @param startPosition [startPosition,startPosition+models.size-1)
     */
    fun replaceRange(models: Iterable<Model>, startPosition: Int, isNotify: Boolean = true): Boolean {
        if (startPosition < 0 || startPosition > this.models.size - 1) {
            return false
        }
        if (!models.iterator().hasNext()) {
            return false
        }
        var size = 0
        models.forEachIndexed { index, model ->
            this.models.removeAt(startPosition + index)
            this.models.add(startPosition + index, model)
            size++
        }
        restPosition()
        if (isNotify) {
            notifyItemRangeChanged(startPosition, size)
        }
        return true
    }

    fun replaceAll(models: Iterable<Model>, isNotify: Boolean = true) {
        this.models.clear()
        this.models.addAll(models)
        restPosition()
        if (isNotify) {
            notifyDataSetChanged()
        }
    }

    fun remove(position: Int, isNotify: Boolean = true): Boolean {
        val size = models.size
        if (position < 0 || position >= size) {
            return false
        }
        models.removeAt(position)
        restPosition()
        if (isNotify) {
            notifyItemRemoved(position)
        }
        return true
    }

    fun removeFirst(isNotify: Boolean = true): Boolean =
            remove(0, isNotify)

    fun removeLast(isNotify: Boolean = true): Boolean =
            remove(models.size - 1, isNotify)

    fun removeRange(start: Int, end: Int, isNotify: Boolean = true): Boolean {
        if (start < 0 || end > models.size || start > end) {
            return false
        }
        var position = end - 1
        repeat(end - start) {
            models.removeAt(position--)
        }
        restPosition()
        if (isNotify) {
            notifyItemRangeRemoved(start, end)
        }
        return true
    }

    fun removeAll(isNotify: Boolean = true) {
        models.clear()
        restPosition()
        if (isNotify) {
            notifyDataSetChanged()
        }
    }

    private fun restPosition(data: MutableList<Model>? = null) {
        (data ?: models).forEachIndexed { index, model ->
            if (index != model.position.get()) {
                model.position.set(index)
            }
        }
    }

    /**
     * ViewHolder唯一标识
     * @param clazz VHModel实际类型
     * @param layoutResId VHModel对应的xml布局id
     */
    private data class VHUnique(val clazz: String, val layoutResId: Int) {
        override fun equals(other: Any?): Boolean {
            if (other is VHUnique) {
                return other.clazz == clazz && other.layoutResId == layoutResId
            }
            return false
        }

        override fun hashCode(): Int {
            return clazz.hashCode() + layoutResId.hashCode()
        }
    }
}