package com.fry.base.utils;

import com.fry.base.func.Func1;
import com.fry.base.global.Constants;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.Observer;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Function;
import me.goldze.mvvmhabit.utils.StringUtils;

/**
 * 2019-09-27 created
 *
 * @author KOM
 */
public class DateUtils {

    public @interface CountdownType {
        /**
         * 00
         */
        int SECOND = 1;
        /**
         * 00:00
         */
        int MINUTE = 2;
        /**
         * 00:00:00
         */
        int HOUR = 3;
        /**
         * 00天00时00分
         */
        int DAY = 4;
        /**
         * 00天 00:00:00
         */
        int DAY_SECOND = 5;
    }

    public @interface CountdownSeparate {
        /**
         * 00:00:00
         */
        int EN = 1;
        /**
         * 00天00时00分
         */
        int CN = 2;
    }

    /***
     * 倒计时转换
     * @param mill 毫秒
     */
    public static String conversionCountdown(long mill){
        @CountdownType int type;
        if(mill >= 24 * 60 * 60 * 1000){
            //超过一天
            type = CountdownType.DAY;
        }else if(mill >= 60 * 60 * 1000){
            //超过小时
            type = CountdownType.HOUR;
        }else {
            //超过一分钟
            type = CountdownType.MINUTE;
        }
        return conversionCountdown(mill / 1000,type,CountdownSeparate.CN);
    }

    /**
     * 转换倒计时
     *
     * @param second 总剩余秒数
     * @return 返回倒计时格式字符串
     */
    public static String conversionCountdown(long second, @CountdownType int type, @CountdownSeparate int separate) {
        switch (type) {
            case CountdownType.HOUR:
                return String.format(separate == CountdownSeparate.EN ? "%s:%s:%s" : "%s时%s分%s秒",
                        patchZero((int) second / 3600),
                        patchZero((int) second % 3600 / 60),
                        patchZero((int) second % 60));
            case CountdownType.MINUTE:
                return String.format(separate == CountdownSeparate.CN ? "%s分%s秒" : "%s:%s",
                        patchZero((int) second / 60),
                        patchZero((int) second % 60));
            case CountdownType.DAY:
                return String.format("%s天%s时%s分",
                        patchZero((int) second / (3600 * 24)),
                        patchZero((int) second % (3600 * 24) / 3600),
                        patchZero((int) second % (3600 * 24) % 3600 / 60));
            case CountdownType.DAY_SECOND:
                return String.format("%s天 %s:%s:%s",
                        patchZero((int) second / (3600 * 24)),
                        patchZero((int) second % (3600 * 24) / 3600),
                        patchZero((int) second % (3600 * 24) % 3600 / 60),
                        patchZero((int) second % 60)
                );
            default:
            case CountdownType.SECOND:
                return separate == CountdownSeparate.CN ? second + "秒" : String.valueOf(second);
        }
    }

    private static String patchZero(int number) {
        return number < 10 ? "0" + number : String.valueOf(number);
    }


    public @interface Format {
        String YMD = "yyyyMMdd";
        String YMD_ = "yyyy-MM-dd";
        String YM_DOT = "yyyy.MM";
        String YMD_DOT = "yyyy.MM.dd";
        String YMD_DOT_HM = "yyyy.MM.dd HH:mm";
        String YMD_SLASH = "yyyy/MM/dd";
        String MD = "MM-dd";
        String MD_HM = "MM-dd HH:mm";
        String YMD_HMS = "yyyy-MM-dd HH:mm:ss";
        String YMD_HMS_SSSS = "yyyy-MM-dd HH:mm:ss SSSS";
        String HM = "HH:mm";
        String HM_CN = "HH时mm分";
        String HMS = "HH:mm:ss";
    }

    public static String format(String time, @Format String format) {
        return format(StringUtils.parseLong(time), format);
    }

    public static String format(long time, @Format String format) {
        try {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat(format, Locale.CHINA);
            return simpleDateFormat.format(new Date(time));
        } catch (NumberFormatException e) {
            e.printStackTrace();
            return "";
        }
    }

    public static String formatForSecond(long second, @Format String format) {
        return format(second * 1000, format);
    }

    public static String formatForSecond(String second, @Format String format) {
        return formatForSecond(StringUtils.parseLong(second), format);
    }

    public static Date parse(String formatDate, @Format String format) {
        try {
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat(format, Locale.CHINA);
            return simpleDateFormat.parse(formatDate);
        } catch (ParseException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static boolean isTimeEmpty(String time) {
        return StringUtils.isEmpty(time) || "0".equals(time);
    }

    /**
     * @param remainingTime 剩余时间 单位秒
     * @param onChange      每秒回调
     * @param onEnd         倒计时正常结束时回调
     * @param onSubscribe   订阅
     */
    public static void countDownTime(int remainingTime, TimeUnit unit,
                                     Func1<Integer> onChange,
                                     Func1<Integer> onEnd,
                                     Func1<Disposable> onSubscribe) {
        if (remainingTime <= 0) {
            if (onChange != null) {
                onChange.run(0);
            }
            return;
        }
        if (onChange != null) {
            onChange.run(remainingTime);
        }
        Observable.zip(Observable.range(1, remainingTime), Observable.interval(1, unit),
                (integer, aLong) -> integer)
                .observeOn(AndroidSchedulers.mainThread())
                .safeSubscribe(new Observer<Integer>() {
                    @Override
                    public void onSubscribe(Disposable d) {
                        if (onSubscribe != null) {
                            onSubscribe.run(d);
                        }
                    }

                    @Override
                    public void onNext(Integer value) {
                        int curReamingTime = remainingTime - value;
                        if (onChange != null) {
                            onChange.run(curReamingTime);
                        }
                        if (curReamingTime == 0 && onEnd != null) {
                            onEnd.run(curReamingTime);
                        }
                    }

                    @Override
                    public void onError(Throwable e) {

                    }

                    @Override
                    public void onComplete() {
                    }
                });
    }

    public static Observable<Integer> countdownTime(int remainingTime, TimeUnit timeUnit) {
        if (remainingTime < 0) {
            remainingTime = 0;
        }
        final int time = remainingTime;
        return Observable.interval(0, 1, timeUnit)
                .observeOn(AndroidSchedulers.mainThread())
                .map(new Function<Long, Integer>() {
                    @Override
                    public Integer apply(Long aLong) throws Exception {
                        return time - aLong.intValue();
                    }
                }).take(time + 1);
    }
    /**
     * 获取时间戳
     * 根据本机时间加上与服务器的差值
     */
    public static long getSystemTimestamp() {
        if (Constants.isNeedRecalculateTimeDiff()) {
            return System.currentTimeMillis();
        }
        return System.currentTimeMillis() + Constants.getTimeDiffForLocalAndService();
    }
}
