package com.fry.base.utils;

import android.graphics.Canvas;
import android.graphics.DashPathEffect;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Rect;
import android.text.TextPaint;

/**
 * 2019-09-29 created
 *
 * @author KOM
 */
public class DrawUtils {

    private DrawUtils() {
    }

    public static TextPaint textPaintInitOrRest(TextPaint textPaint, int spTestSize, int textColor) {
        if (textPaint == null)
            textPaint = new TextPaint();
        textPaint.setAntiAlias(true);
        textPaint.setTextSize(DensityUtils.sp2px(spTestSize));
        textPaint.setColor(textColor);
        return textPaint;
    }

    public static <T extends Paint> void dashPathInitOrRest(T paint, boolean isInit, float... intervals) {
        if (isInit) {
            DashPathEffect pathEffect = new DashPathEffect(intervals, 1);
            paint.setPathEffect(pathEffect);
        } else {
            paint.setPathEffect(null);
        }
    }

    /**
     * 在targetRect区域中居中画text
     *
     * @param canvas     画布
     * @param targetRect 矩形
     * @param text       文本
     * @param paint      画笔
     */
    public static void drawTextCenterForRect(Canvas canvas, Rect targetRect, String text, Paint paint) {
        Paint.FontMetricsInt fontMetrics = paint.getFontMetricsInt();
        int baseline = (targetRect.bottom + targetRect.top - fontMetrics.bottom - fontMetrics.top) / 2;
        // 下面这行是实现水平居中，drawText对应改为传入targetRect.centerX()
        paint.setTextAlign(Paint.Align.CENTER);
        canvas.drawText(text,
                targetRect.centerX(), baseline, paint);
    }

    /**
     * 画箭头
     *
     * @param canvas 画布
     * @param cx     箭头线的中心点X坐标
     * @param cy     箭头线的中心点X坐标
     * @param length 箭头的长度
     * @param angle  箭头的角度 0 水平向右 90 水平向下
     * @param paint  画笔
     */
    public static void drawArrow(Canvas canvas, int cx, int cy,
                                 int length, int angle, Paint paint) {

        Paint.Cap defCap = paint.getStrokeCap();
        paint.setStrokeCap(Paint.Cap.ROUND);

        int halfLength = length / 2;

        //画相对画布 向下的箭头
        int layer1 = canvas.save();
        canvas.translate(cx, cy);

        int layer2 = canvas.save();
        canvas.rotate(angle);

        canvas.drawLine(-halfLength, 0, halfLength, 0, paint);
        canvas.translate((float) (halfLength), 0);
        canvas.rotate(135);
        canvas.drawLine(0, 0, halfLength, 0, paint);
        canvas.rotate(90);
        canvas.drawLine(0, 0, halfLength, 0, paint);

        canvas.restoreToCount(layer2);

        canvas.restoreToCount(layer1);

        paint.setStrokeCap(defCap);


    }

    /**
     * 画等于号
     *
     * @param canvas       画布
     * @param cx           等于号的中心点X坐标
     * @param cy           等于号的中心点X坐标
     * @param equalLength  等于号的长度
     * @param equalPadding 等于号之间的padding
     * @param paint        画笔
     */
    public static void drawEqual(Canvas canvas, int cx, int cy,
                                 int equalLength, int equalPadding, Paint paint) {
        Paint.Cap defCap = paint.getStrokeCap();
        paint.setStrokeCap(Paint.Cap.ROUND);

        int halfLength = equalLength / 2;

        int layer1 = canvas.save();

        canvas.translate(cx, cy - equalPadding / 2 - paint.getStrokeWidth());
        canvas.drawLine(-halfLength, 0, halfLength, 0, paint);

        canvas.restoreToCount(layer1);

        int layer2 = canvas.save();

        canvas.translate(cx, cy + equalPadding / 2 + paint.getStrokeWidth());
        canvas.drawLine(-halfLength, 0, halfLength, 0, paint);

        canvas.restoreToCount(layer2);

        paint.setStrokeCap(defCap);
    }

    /**
     * 画三角形
     * @param canvas 画布
     * @param cx 中心点 X
     * @param cy 中心点 Y
     * @param sideLength 边长
     * @param paint 画笔
     */
    public static void drawTriangle(Canvas canvas, int cx, int cy,
                                    int sideLength, Paint paint) {
        int x = sideLength / 10;
        int halfLength = sideLength / 2;
        Path path = new Path();
        path.moveTo(cx - halfLength + x, cy - halfLength);
        path.lineTo(cx + halfLength + x, cy);
        path.lineTo(cx - halfLength + x, cy + halfLength);
        path.close();
        canvas.drawPath(path, paint);
    }

    public static int[] calculateTextWidthAndHeight(String text,Paint paint){
        Paint.FontMetrics fontMetrics = paint.getFontMetrics();
        int fontHeight = (int) (fontMetrics.bottom - fontMetrics.top);
        int fontWidth = (int) paint.measureText(text);
        return new int[]{fontWidth,fontHeight};
    }

}
