package com.fry.base.utils;

import android.graphics.Shader;
import android.graphics.drawable.Drawable;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.TextPaint;
import android.text.TextUtils;
import android.text.style.AbsoluteSizeSpan;
import android.text.style.ForegroundColorSpan;

import androidx.annotation.ColorInt;
import androidx.annotation.DrawableRes;

import com.fry.base.ui.LinearGradientFontSpan;

import java.util.List;

/**
 * 2019-09-26 created
 *
 * @author KOM
 */
public class SpannableUtils {

    /**
     * 高亮指定文本
     *
     * @param content        全部内容
     * @param color          高亮的颜色
     * @param highLightTexts 内容中需高亮的文本
     * @return Spannable
     */
    public static Spannable highLightText(String content, String[] highLightTexts, @ColorInt int[] color) {
        if (!TextUtils.isEmpty(content) && highLightTexts.length > 0) {
            SpannableStringBuilder style = new SpannableStringBuilder(content);
            int start = 0;
            int end = 0;
            String highLightText = "";
            for (int i = 0, size = highLightTexts.length; i < size; i++) {
                highLightText = highLightTexts[i];
                if (!TextUtils.isEmpty(highLightText)) {
                    if (content.contains(highLightText)) {
                        start = content.indexOf(highLightText);
                        end = start + highLightText.length();
                    }
                    style.setSpan(new ForegroundColorSpan(color[i]), start, end, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
                }
            }
            return style;
        }
        return new SpannableString(TextUtils.isEmpty(content) ? "" : content);
    }

    public static Spannable highLightText(String content, String highLightText, @ColorInt int color) {
        return highLightText(content, new String[]{highLightText}, new int[]{color});
    }

    /**
     * 添加图片到文字中
     */
    public static SpannableString addDrawable(@DrawableRes int iconResId, String content, float textSize) {
        SpannableString sp = new SpannableString("  " + content);
        TextPaint textPaint = new TextPaint();
        textPaint.setTextSize(textSize);
        int[] textW_H = DrawUtils.calculateTextWidthAndHeight("测试文字行高", textPaint);

        Drawable drawable = ResourceUtils.getDrawable(iconResId);
        int width = drawable.getIntrinsicWidth();
        int height = drawable.getIntrinsicHeight();
        float scale = (float) textW_H[1] / height;
        width = (int) (width * scale * 0.8);
        height = (int) (textW_H[1] * 0.8);
        drawable.setBounds(0, 0, width, height);
        //普通imageSpan 做对比
        VerticalAlignImageSpan imageSpan2 = new VerticalAlignImageSpan(drawable);

        sp.setSpan(imageSpan2, 0, 1, SpannableString.SPAN_INCLUSIVE_EXCLUSIVE);

        return sp;
    }

    /**
     * 添加图片到文字中
     */
    public static CharSequence addDrawables(@DrawableRes List<Integer> iconResIds, CharSequence content, float textSize) {
        SpannableStringBuilder spBuilder = new SpannableStringBuilder();
        TextPaint textPaint = new TextPaint();
        textPaint.setTextSize(textSize);
        int[] textW_H = DrawUtils.calculateTextWidthAndHeight("测试文字行高", textPaint);
        if (iconResIds != null && iconResIds.size() > 0) {
            SpannableString spannableString = null;
            int iconResId = 0;
            for (int p = 0; p < iconResIds.size(); p++) {
                iconResId = iconResIds.get(p);
                spannableString = new SpannableString("  ");
                Drawable drawable = ResourceUtils.getDrawable(iconResId);
                int width = drawable.getIntrinsicWidth();
                int height = drawable.getIntrinsicHeight();
                float scale = (float) textW_H[1] / height;
                width = (int) (width * scale * 0.8);
                height = (int) (textW_H[1] * 0.8);
                drawable.setBounds(0, 0, width, height);
                //普通imageSpan 做对比
                VerticalAlignImageSpan imageSpan2 = new VerticalAlignImageSpan(drawable);

                spannableString.setSpan(imageSpan2, 0, 1, SpannableString.SPAN_INCLUSIVE_EXCLUSIVE);
                spBuilder.append(spannableString);
            }
        }
        spBuilder.append(content);
        return spBuilder;
    }

    /***
     * 获取富文本信息
     * 三个参数长度必须一致
     * @param content 文本内容
     * @param size 大小 像素
     * @param color 颜色
     */
    public static Spannable getSpanTxt(String[] content, int[] size, @ColorInt int[] color) {
        if (content == null || size == null || color == null) {
            return new SpannableString("");
        }
        //如果三者长度不统一，直接返回
        if ((content.length + size.length + color.length) % 3 != 0) {
            return new SpannableString("");
        }
        //因为三者长度一致，所以只判断一个即可
        if(content.length == 0){
            return new SpannableString("");
        }
        SpannableStringBuilder builder = new SpannableStringBuilder();
        SpannableString tmp;

        for (int i = 0;i<content.length;i++){
            tmp = new SpannableString(content[i]);
            tmp.setSpan(new ForegroundColorSpan(color[i]),0,tmp.length(), Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
            tmp.setSpan(new AbsoluteSizeSpan(size[i]),0,tmp.length(), Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
            builder.append(tmp);
        }
        return builder;

    }
    /***
     * 获取富文本信息
     * 2个参数长度必须一致
     * @param content 文本内容
     * @param size 大小 像素
     */
    public static Spannable getSpanTxt(String[] content, int[] size) {
        if (content == null || size == null ) {
            return new SpannableString("");
        }
        //如果三者长度不统一，直接返回
        if(content.length == 0 || content.length != size.length){
            return new SpannableString("");
        }

        SpannableStringBuilder builder = new SpannableStringBuilder();
        SpannableString tmp;

        for (int i = 0;i<content.length;i++){
            tmp = new SpannableString(content[i]);
            tmp.setSpan(new AbsoluteSizeSpan(size[i]),0,tmp.length(), Spanned.SPAN_INCLUSIVE_EXCLUSIVE);
            builder.append(tmp);
        }
        return builder;
    }

    /***
     * 获取渐变文案
     * @param content
     * @param colors 所有颜色值
     * @param points 颜色渐变点，可以为null，如果为null则平滑绘制
     * @return
     */
    public static Spannable getGradientTxt(String content, int[] colors, float[] points){
        if(TextUtils.isEmpty(content)){
            return new SpannableString("");
        }
        SpannableStringBuilder spannableStringBuilder = new SpannableStringBuilder(content);
        LinearGradientFontSpan span = new LinearGradientFontSpan(colors, points,Shader.TileMode.REPEAT);
        spannableStringBuilder.setSpan(span, 0, spannableStringBuilder.length(), Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
        return spannableStringBuilder;
    }

}
