package com.fry.base.widget;

import android.content.Context;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.widget.FrameLayout;

import androidx.customview.widget.ViewDragHelper;

import me.goldze.mvvmhabit.utils.KLog;

/**
 * 2019-12-15 created
 *
 * @author KOM
 */
public class DrawerDragLayout extends FrameLayout {

    public enum State {
        OPEN, CLOSE
    }

    public interface DrawerListener {

        /**
         * @param slideOffset [0,1]
         */
        void onDrawerSlide(View drawerView, float slideOffset);

        void onDrawerOpened(View drawerView);

        void onDrawerClosed(View drawerView);

        /**
         * @param newState ViewDragHelper.STATE_IDLE
         *                 ViewDragHelper.STATE_DRAGGING
         *                 ViewDragHelper.STATE_SETTLING
         */
        void onDrawerStateChanged(int newState);
    }

    private static String TAG = DrawerDragLayout.class.getSimpleName();

    private ViewDragHelper mDragHelper;
    /**
     * 拖动的能力
     */
    private boolean dragEnable = true;
    /**
     * 拖动关闭的能力
     */
    private boolean dragCloseEnable = true;

    private State state = State.OPEN;

    private View drawerView;
    private int leftViewWidth;

    private DrawerListener drawerListener;

    public DrawerDragLayout(Context context) {
        this(context, null);
    }

    public DrawerDragLayout(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public DrawerDragLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }


    @Override
    protected void onFinishInflate() {
        super.onFinishInflate();
        drawerView = getChildAt(1);
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
        leftViewWidth = drawerView.getMeasuredWidth();
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        final int childCount = getChildCount();
        for (int i = 0; i < childCount; i++) {
            final View child = getChildAt(i);

            if (child.getVisibility() == GONE) {
                continue;
            }
            final FrameLayout.LayoutParams lp = (FrameLayout.LayoutParams) child.getLayoutParams();
            int childWidth = child.getMeasuredWidth();
            int childHeight = child.getMeasuredHeight();
            if (child == drawerView) {
                boolean isOpen = state == State.OPEN;
                int l = isOpen ? lp.leftMargin : -childWidth;
                int r = isOpen ? lp.leftMargin + childWidth : 0;
                child.layout(l, lp.topMargin, r,
                        lp.topMargin + childHeight);
            } else {
                child.layout(lp.leftMargin, lp.topMargin,
                        lp.leftMargin + childWidth,
                        lp.topMargin + childHeight);
            }
        }
    }

    private void init() {

        mDragHelper = ViewDragHelper.create(this, 1f, new ViewDragHelper.Callback() {

            @Override
            public boolean tryCaptureView(View child, int pointerId) {
                return dragEnable && drawerView == child && (state == State.CLOSE || dragCloseEnable);
            }

            @Override
            public void onEdgeTouched(int edgeFlags, int pointerId) {
                super.onEdgeTouched(edgeFlags, pointerId);
                mDragHelper.captureChildView(drawerView, pointerId);
            }

            @Override
            public int getViewHorizontalDragRange(View child) {
                return state == State.CLOSE ? child.getWidth() / 4 : child.getWidth() / 4;
            }

            @Override
            public void onViewPositionChanged(View changedView, int left, int top, int dx, int dy) {
                KLog.d(TAG, "onViewPositionChanged changedView:" + changedView +
                        " left:" + left);
                if (drawerListener == null) return;
                int offset = changedView.getWidth() + left;
                drawerListener.onDrawerSlide(drawerView,
                        (float) offset / changedView.getWidth());
            }

            /**
             * (计算)固定拖动View在水平方向上被改变后的位置
             * @param child
             * @param left
             * @param dx
             * @return
             */
            @Override
            public int clampViewPositionHorizontal(View child, int left, int dx) {
                KLog.d(TAG, "clampViewPositionHorizontal child:" + child +
                        " left:" + left);
                return Math.min(Math.max(left, -leftViewWidth), 0);
            }

            /**
             * 手指释放时 被调用
             * @param releasedChild 拖动的View
             * @param xVel x方向速度
             * @param yVel y方向速度
             */
            @Override
            public void onViewReleased(View releasedChild, float xVel, float yVel) {
                int left = releasedChild.getLeft();
                int startLeft = 0;
                int endLeft = 0;
                State endState;
                switch (state) {
                    case OPEN:
                        endLeft = -leftViewWidth;
                        endState = State.CLOSE;
                        break;
                    default:
                    case CLOSE:
                        startLeft = -leftViewWidth;
                        endState = State.OPEN;
                        break;
                }
                int dLeft = Math.abs(startLeft - left);
                KLog.d(TAG, "startLeft:" + startLeft
                        + " endLeft:" + endLeft
                        + " left:" + left
                        + " endState:" + endState);
                if (dLeft >= leftViewWidth / 3) {
                    final State changeState = endState;
                    mDragHelper.settleCapturedViewAt(endLeft, releasedChild.getTop());
                    post(new Runnable() {
                        @Override
                        public void run() {
                            changeState(changeState);
                        }
                    });
                } else {
                    mDragHelper.settleCapturedViewAt(startLeft, releasedChild.getTop());
                }
                invalidate();
            }

            @Override
            public void onViewDragStateChanged(int state) {
                super.onViewDragStateChanged(state);
                if (drawerListener == null) return;
                drawerListener.onDrawerStateChanged(state);
            }
        });
        changeState(state);
    }

    public boolean isDrawerOpen() {
        return state == State.OPEN;
    }

    public void closeDrawer() {
        if (drawerView == null) return;
        changeState(State.CLOSE);
        mDragHelper.smoothSlideViewTo(drawerView, -leftViewWidth, drawerView.getTop());
        invalidate();
    }

    public void openDrawer() {
        if (drawerView == null) return;
        changeState(State.OPEN);
        mDragHelper.smoothSlideViewTo(drawerView, 0, drawerView.getTop());
        invalidate();
    }

    private void changeState(State state) {
        if (mDragHelper == null) return;
        DrawerDragLayout.this.state = state;
        mDragHelper.setEdgeTrackingEnabled(
                DrawerDragLayout.this.state == State.CLOSE
                        ? ViewDragHelper.EDGE_LEFT
                        : dragCloseEnable ? ViewDragHelper.EDGE_RIGHT : 0);
        if (drawerListener != null) {
            switch (state) {
                case CLOSE:
                    drawerListener.onDrawerClosed(drawerView);
                    break;
                case OPEN:
                    drawerListener.onDrawerOpened(drawerView);
                    break;
            }
        }
    }

    public boolean isDragEnable() {
        return dragEnable;
    }

    public void setDragEnable(boolean dragEnable) {
        this.dragEnable = dragEnable;
    }

    public boolean isDragCloseEnable() {
        return dragCloseEnable;
    }

    public void setDragCloseEnable(boolean dragCloseEnable) {
        this.dragCloseEnable = dragCloseEnable;
    }

    @Override
    public boolean onInterceptTouchEvent(MotionEvent event) {
        if (dragEnable) {
            return mDragHelper.shouldInterceptTouchEvent(event);
        }
        return super.onInterceptTouchEvent(event);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (dragEnable) {
            mDragHelper.processTouchEvent(event);
            return true;
        }
        return super.onTouchEvent(event);
    }

    @Override
    public void computeScroll() {
        super.computeScroll();
        if (dragEnable
                && mDragHelper.continueSettling(true)) {
            invalidate();
        }
    }

    public DrawerListener getDrawerListener() {
        return drawerListener;
    }

    public void setDrawerListener(DrawerListener drawerListener) {
        this.drawerListener = drawerListener;
    }
}